!******************************************************************************
!*                                                                            *
!*                               CDVENSAT                                     *
!*                                                                            *	
!*       Programa para conversao de formato de dados de vento obtidos por     *
!*                   por satelites produzidos no DSA                          *
!*                                                                            *
!*              CENTRO DE PREVISAO DO TEMPO E ESTUDOS CLIMATICOS              *
!*                           (MCT-CPTE-INPE)                                  *
!*                                                                            *
!*             GRUPO DE PRE-PROCESSAMENTO DE DADOS  METEOROLOGICOS            *
!******************************************************************************
!  
!   Formatos de Entrada
!         tabela texto do DSA
!
!   
! 	Formatos de Saida: 
!       1 - GRADS
!       2 - BUFR
!       3  -Texto para gempek
!
!      
!  HISTORICO : 	
!
!	VERSAO 0.1 JUNHO 2004 - SERGIO H.S.FERREIRA 
!
!                  Versao inicial com a inclusao dos modulos de leitura de tabelas
!                  textp e modulo de gravacao GRADS (MGRADS)
!
!   VERSAO 0.4 25/07/2004 - SERGIO H.S.FERREIRA 
!          
!                  Inclusao do Modulo de gravacao em formato BUFR (MBUFR)
!
!----------------------------------------------------------------------------
!


program CDVENSAT

  USE STRINGFLIB
  USE MREADTBL
  USE DATELIB
  USE MGRADS
  USE mgempak
  !USE MSFLIB  ! Para compilacao em Windows ( Microsoft Power Station )
  USE MBUFR


  implicit none			! Nada de definicoes implicitas !

  !{ DECLARACAO DAS VARIAVEIS 
  	
  	character(len=255) ::infile        ! nome de um arquivo de entrada
 	character(len=255) ::lista         ! nome da lista de arquivos de entrada
 	character(len=255) ::outfile       ! nome do arquivo de saÃ­da sem extensÃ£o
        character(len=255) ::outfilex      ! nome do arquivo de saÃ­da com extensÃ£o
	character(len=255) ::option             
  	character(len=14)  ::indate        ! Data de entrada
        character(len=13)  ::aux
	integer(2) 	       ::status,i,argc ! Utilizadas na leitura dos arquivos
  	integer  	       ::iargc
 	integer            ::j,un    
  	integer            ::nobs          ! Numero de linhas de obs
        integer            ::nscols        ! Numero de colunas de obs        
	real*8             ::cdate         ! Data dos dados lidos
	real               ::CX,CY         ! Variaveis auxiliares 	      
   	type(mbufr_id)     ::station_id	   ! Identificacao da estacao p/ 
	                                   ! uso na secao 2 do BUFR
	type(mbufr_date)   ::Hobs          ! Hora da Observacao p/ uso 
	                                   ! nas secoes 1 e/ou 2 do BUFR
	character(len=255) :: canal        ! Codigo do canal utilizado IR OU MV
        integer ::icanal                   ! codigo BUFR do canal utilizado 
                                           !conforme 1.2 code/frag table-7
                                           ! 1 - IR  3 - WV
        real :: aux2
!}

!{  DIMENSIONAMENTO DE MATRIZES

    character(len=13), dimension(10000  )::gdate  !Data para gempak matrix obs
    character(len=4),  dimension(      7)::tblcod !pseudo codigo BUFR de cada coluna de obs
    character(len=30), dimension(      7)::tbldes !Descricao do dado de cada coluna
    type(stidtype),    dimension(10000  )::STID   !Identificacao do Pto de Obs p/ grads
    REAL,              dimension(10000,7)::obs    !Matriz de dados     
    integer,           dimension(      5)::tblcol !associa colunas do arquivo com obs
    logical,           dimension(      3)::grava
    character(len=255),dimension(      3)::FLAG 
    type(descbufr)    ,dimension(     12)::D      ! Descritores da tabela bufr
    REAL              ,dimension(100, 12)::V      ! Matriz de dados auxiliar, para 
    integer           ,dimension(100, 12)::Iconf                                              ! passagem dos dados para o MBUFR
 
  !}    



  !{ INICIALIZACAO DAS VARIAVEIS 
       
  	argc=0
	infile=''
  	outfile=''
        grava(1:3)=.false.
        indate=''
        nscols=5
!      Colunas         Psedo codigo     descricao
!      ###########   ################  #####################################
       tblcol(1)=6 ; tblcod(1)="SLAT"; tbldes(1)="Latitude  "
       tblcol(2)=7 ; tblcod(2)="SLON"; tbldes(2)="Longitude"
       tblcol(3)=8 ; tblcod(3)="WDSP"; tbldes(3)="Direcao do Vento (g)" 
       tblcol(4)=9 ; tblcod(4)="WDIR"; tbldes(4)="Velocidade do Vento(m/s)"
       tblcol(5)=11; tblcod(5)="PRLC"; tbldes(5)="Nivel Isobarico (hPa)"
  
       
     
  !}
      
  !{ Pega os argumentos de Entrada: Data e Nomes dos arquivos de entrada e saida
 	argc =  iargc()	
  	if (argc>4) then 
           i=argc-0;call GetArg(i,outfile)
           i=argc-1;call GetArg(i,infile)  
           i=argc-2;call GetArg(i,indate)
           i=argc-3;call GetArg(i,canal)
           cdate=fjulian(indate)	        
  	endif
  !}


  !{ Processa Opcoes 
 
      FLAG(1) = '-bufr'
      FLAG(2) = '-grads'
      FLAG(3) = '-txt'
  
  	DO i=1,3
      	   DO j=1, argc-4
             call GetArg(j,option)
              if (option==FLAG(i)) grava(i)=.true.
           ENDDO
        ENDDO
   
   icanal=15 ! Valor MISSING
   if (canal=="WV") icanal=3
   if (canal=="IR") icanal=1
  !}

 !{ Parametros de teste
 !  grava(1)=.true.
 !  grava(2)=.true.
 !  infile="dados_exemplo/GwindL_C_0401231115_1110.txt"
 !  outfile="saida"
 !  cdate=fjulian("200404010100")
 !  argc=5
 !  icanal=1
 !}

  !{ APRESENTACAO DO PROGRAMA 
       print *,'_______________________________________________________________________'
       PRInt *,'CDVENSAT -PROGRAMA DE CONVERSAO DE FORMATO DADOS DE VENTO POR SATELITE ' 
       PRINT *,'             Versao 0.8 - GDM-CPTEC/INPE Julho 2004'
       print *,'_______________________________________________________________________' 
       PRINT *,' '
       PRINT *,' '    

       if  (argc<5) then             
         print *,' '
         print *,' Use:'
         print *,' > cdvensat.x options date channel input_name output_name '
         print *,'    options: -grads -bufr -txt'
         print *,'    channel: (IR or WV)'
         print *,'    date: YYYYMMDDHHmm'	
         print *,'    input_name: Arquivo de entrada'
         print *,'    output_name: Arquivo de saida sem extensao'
         print *,''
         stop
       else 
         print *,'INPUT           ', trim(infile)
         print *,'OUTPUT          ', trim(outfile)
         print *,'CANAL',icanal,trim(canal)
         print *,'BUFR  Y(1) N(0) ', grava(1)
         print *,'GRADS Y(1) N(0) ', grava(2)
         print *,'TEXTO Y(1) N(0) ', grava(3)
         print *, grdate(cdate)
       end if
!}



!{ LEITURA DOS DADOS

    call conf_mreadtbl(15,nscols,tblcol)   ! Configura mreadtbl para leitura
    call readtbl(10,infile,obs,nobs)       ! ler os dados utilizando mreadtbl
    print *,"Numero de dados Lidos:",nobs
!}

!{ GRAVACAO DOS DADOS EM FORMATO GRADS
  if (grava(2)) then 


   !{ Conversoes de dados

   do i=1,nobs
    !{ atribuicao dos valores de latitude e longitue 
      STID(I)%COD="SATOBDSA"
      STID(I)%LAT=obs(I,1)
      STID(I)%LON=obs(I,2)
    !}
    !{ Conversao de direcao e velocidade em componentes X e Y
    
       obs(i,6) = -obs(i,3) * SIND(obs(i,4))
       obs(i,7) = -obs(i,3) * COSD(obs(i,4))
       tblcod(6)="UCMP"; tbldes(6)="Componente U do vento (m/s)"
       tblcod(7)="VCMP"; tbldes(7)="Componente V do vento (m/s)"
    !}
   end do
   
   !}
 
      
   
   !{Salvar CTL e BINARIO
      call savectl(3,outfile,cdate,tblcod,tbldes,7)    
      call savebin(3,outfile,obs,nobs,STID,7)
   !}



      
  end if
!}



!{GRAVACAO DOS DADOS EM FORMATO TEXTO P/  GEMPAK
  if (grava(3)) then
  outfilex=trim(outfile)//".txt"   
   
    
  write(aux,'(i4,2i2.2,"/",2i2.2)')year(cdate),month(cdate),day(cdate),hour(cdate),0
  gdate(1:nobs)=aux     
  
 
  call save_mgempak(10,outfilex,obs,nobs,tblcod,nscols,gdate)



 end if   
 
      

!}


!{ GRAVACAO DOS DADOS EM FORMATO BUFR
  if (grava(1)) then
   outfilex=trim(outfile)//".rdb"
   print *,"Gravando ",trim(outfilex)
   	 !Preparar para salvar formato BUFR
     !{
	 
	   !Abrir arquivo BUFR e inicializar os configuraçoes
	   !gerais: 
	   !   3)Centro gerados (INPE) = 45
	   !   4)Gravação da Secao 2   = .false.
	   !{

		call open_mbufr(15,outfilex,46)
	
	  !}
	  !{ Dados de Secao 1 
	   
	        Hobs%ano=year(cdate)
		Hobs%mes=month(cdate)
		Hobs%dia=day(cdate)
		Hobs%hora=hour(cdate)
		Hobs%min=0

	   !}

	   ! A secao 2 nao esta sendo utilizada, portanto 
	   ! portanto as linhas a seguir estam comentadas
	   
	   !Station_Id%cod=""   ! Codigo da Estacao
	   !Station_Id%name=""  ! Nome da Estacao
	   !Station_Id%lat=0.0  ! Latitude da Estacao
	   !Station_Id%lon=0.0  ! Longitude da Estacao


      !{ Descritores que serao utilizados  (Secao 3) 
	   
	   D(1:10)%F=0 ! Todos os elementos sao da tabela B
	   D( 1)%x=01; D( 1)%y=007 ! Identificacao do Satelite  
       D( 2)%x=02; D( 2)%y=023 ! Metodo de Calculo de Vento por Satelite
	   D( 3)%x=04; D( 3)%y=001 ! Ano
	   D( 4)%x=04; D( 4)%y=002 ! MES
	   D( 5)%x=04; D( 5)%y=003 ! Dia
	   D( 6)%x=04; D( 6)%y=004 ! Hora
	   D( 7)%x=04; D( 7)%Y=005 ! Minuto
	   D( 8)%x=05; D( 8)%Y=002 ! CLAT
	   D( 9)%x=06; D( 9)%y=002 ! CLON
	   D(10)%x=10; D(10)%y=004 ! Pressao cooedanada vertical (Pa)
	   D(11)%x=11; D(11)%y=001 ! Direcao do vento (graus)
	   D(12)%x=11; D(12)%y=002 ! Velocidade do vento (m/s)
	  !}
	  
	  
	  !Indices de confiabilidade
	  !{
	     Iconf(1:1,1:12)=70 !% Confiabilidade = 70% Para todos os casos
	
	  !}

	  !Conversao de dados/unidade	 (secao 4)
	  ! A posicao j em que os valores são atribuidos a matrix v(i,j)
	  ! deve ser as mesma dos descritores em D(j)
	  ! 
	  !{
	   Do i=1,nobs
                  v(1,1)=256                ! GOES 12
                  v(1,2)=icanal              ! 1-IR  3-WV
	   	  V(1,3)=Hobs%ano	    !-Usa a mesma data da secao 1 em todas
		  v(1,4)=Hobs%mes		! os dados de SATOB 
		  v(1,5)=Hobs%dia
		  v(1,6)=hour(cdate)
		  v(1,7)=0
		  v(1,8)=obs(i,1)        !-Latitude 
		  v(1,9)=obs(i,2)		 !-Longitude
		  v(1,10)=obs(i,5)*100    !-Pressao 
!		  aux2=obs(i,5)
		  v(1,11)=obs(i,4)        !-Direcao do vento
		  v(1,12)=obs(i,3)       !-Velocidade do vento
		  
	  ! {Salvar uma mensagem em BUFR	Type 5 =Sataob, Subtype=0 
	  !  
	  !  Para gravar sem a secao 2 utiliza-se
	  !    call save_mbufr(15,d,12,v,iconf,1,5,0,Hobs)
	  !
	  !  Para gravar com a secao 2 utiliza-se 
	  !		 call save_mbufr(15,d,12,v,iconf,1,5,0,Hobs,Station_Id)
	   
		 call save_mbufr(15,d,12,v,iconf,1,5,0,Hobs)

	   !}


	   end do
	  !}

	


	   ! Encerrar o arquivo 
	   ! {
	    call close_mbufr(15)
	   !}

  end if
!}

END
